//File: 	game.cs
//Author:	Base FPS example file.
//Date:		19 Oct 2005
//Notes:
//
//	19 Oct 2005	MP
//	- Modified to support text based player health and ammo.
//
//	24 Oct 2005 JS
//	- Weather 
//	- Shotgun as starting equipment
//  1 Nov 2005	RCBJ
//  -Added logic to onClientEnterGame() that will prevent more than
//   two clients connecting at one time.  At the moment it causes the
//   client who was disconnected to hang.  Will need to handle this 
//   more gracefully in the future.
//  18 Nov 2005 JS
//	-Added spawner objects (v1.0)
//  28 Nov 2005 TL
//  -Modified starting energy Ammo to 200 for easier check on battery
// Game duration in secs, no limit if the duration is set to 0
$Game::Duration = 20 * 60;

// When a client score reaches this value, the game is ended.
$Game::EndGameScore = 30;

// Pause while looking over the end game screen (in secs)
$Game::EndGamePause = 10;


//-----------------------------------------------------------------------------
//  Functions that implement game-play
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function onServerCreated()
{
   // Server::GameType is sent to the master server.
   // This variable should uniquely identify your game and/or mod.
   $Server::GameType = "FPS Starter Kit";

   // Server::MissionType sent to the master server.  Clients can
   // filter servers based on mission type.
   $Server::MissionType = "Deathmatch";

   // GameStartTime is the sim time the game started. Used to calculated
   // game elapsed time.
   $Game::StartTime = 0;

   // Load up all datablocks, objects etc.  This function is called when
   // a server is constructed.
   exec("./audioProfiles.cs");
   exec("./camera.cs");
   exec("./markers.cs"); 
   exec("./triggers.cs"); 
   exec("./inventory.cs");
   exec("./shapeBase.cs");
   exec("./item.cs");
   exec("./health.cs");
   exec("./staticShape.cs");
   exec("./weapon.cs");
   exec("./radiusDamage.cs");
   exec("./crossbow.cs");
   exec("./player.cs");
   exec("./chimneyfire.cs");
//   exec("./aiPlayer.cs");
   exec("./weather.cs");
   exec("./lightning.cs");
   exec("./machinegun.cs");
   exec("./shotgun.cs");
   exec("./rocketlauncher.cs");
   exec("./pulselaser.cs");
   exec("./antweapon.cs");
   exec("./machinegunupgrade.cs");
   exec("./shotgunupgrade.cs");
   exec("./rocketlauncherupgrade.cs");
   exec("./pulselaserupgrade.cs");
   exec("./laserbeamupgrade.cs");
   exec("./laserbeam.cs");
   
   exec("./Enemies/BallisticAntaiPlayer.cs");
   exec("./Enemies/EnergyAntaiPlayer.cs");
   exec("./Enemies/MeleeAntaiPlayer.cs");
   exec("./Enemies/PoisonAntaiPlayer.cs");
   exec("./Enemies/CorrosionAntaiPlayer.cs");
   exec("./Enemies/NumbAntaiPlayer.cs");
   exec("./Enemies/BeeAIPlayer.cs"); 
   //exec("./NumbAntWeapon.cs");
   //exec("./CorrosionAntWeapon.cs");
   //exec("./PoisonAntWeapon.cs");
   exec("./immunity.cs");
   exec("./damagemultiplier.cs");
   exec("./superarmor.cs");
   exec("./ammo.cs");
   exec("./battery.cs");
   exec("./supershield.cs");
	exec("./level_one.cs");
	exec("./hive.cs");
	exec("./hivefuncs.cs");
	//exec("./spawner.cs");
	exec("./Controller.cs");
	
exec("./targetTeleportTrigger.cs");
   // Keep track of when the game started
   $Game::StartTime = $Sim::Time;
}

function onServerDestroyed()
{
   // This function is called as part of a server shutdown.
}


//-----------------------------------------------------------------------------

function onMissionLoaded()
{
   // Called by loadMission() once the mission is finished loading.
   // Nothing special for now, just start up the game play.
EnemyContainer::buildLevelPFMA();

   startGame();

}

function onMissionEnded()
{
   // Called by endMission(), right before the mission is destroyed

   // Normally the game should be ended first before the next
   // mission is loaded, this is here in case loadMission has been
   // called directly.  The mission will be ended if the server
   // is destroyed, so we only need to cleanup here.
   cancel($Game::Schedule);
   $Game::Running = false;
   $Game::Cycling = false;

   //Make sure the EnemyContainer Singleton is reset properly...
   EnemyContainer::EnemyContainerReset();
}


//-----------------------------------------------------------------------------

function startGame()
{
   if ($Game::Running) {
      error("startGame: End the game first!");
      return;
   }

   // Inform the client we're starting up
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameStart');

      // Other client specific setup..
      %cl.score = 0;
   }

   // Start the game timer
   if ($Game::Duration)
      $Game::Schedule = schedule($Game::Duration * 1000, 0, "onGameDurationEnd" );
   $Game::Running = true;
   
   // Start the AIManager
//   new ScriptObject(AIManager) {};
//   MissionCleanup.add(AIManager);
//   AIManager.think();

	EnemyContainer::EnemyContainerReset();

	BallisticAntAIPlayer::LoadEntities();
	EnergyAntAIPlayer::LoadEntities();
	MeleeAntAIPlayer::LoadEntities();
	PoisonAntAIPlayer::LoadEntities();
	CorrosionAntAIPlayer::LoadEntities();
	NumbAntAIPlayer::LoadEntities();
	echo("Trying to Initialize Level");	 
	EnemyContainer::buildLevelPFMA($Server::MissionFile);  
 
	//schedule(30000,0,LevelInit);
	LevelInit();
}

function endGame()
{
   if (!$Game::Running)  {
      error("endGame: No game running!");
      return;
   }
   
   // Stop the AIManageraa
   AIManager.delete();

   // Stop any game timers
   cancel($Game::Schedule);

   // Inform the client the game is over
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameEnd');
   }

   // Delete all the temporary mission objects
   resetMission();
   $Game::Running = false;
}

function onGameDurationEnd()
{
   // This "redirect" is here so that we can abort the game cycle if
   // the $Game::Duration variable has been cleared, without having
   // to have a function to cancel the schedule.
   if ($Game::Duration && !isObject(EditorGui))
      cycleGame();
}


//-----------------------------------------------------------------------------

function cycleGame()
{
   // This is setup as a schedule so that this function can be called
   // directly from object callbacks.  Object callbacks have to be
   // carefull about invoking server functions that could cause
   // their object to be deleted.
   if (!$Game::Cycling) {
      $Game::Cycling = true;
      $Game::Schedule = schedule(0, 0, "onCycleExec");
   }
}

function onCycleExec()
{
   // End the current game and start another one, we'll pause for a little
   // so the end game victory screen can be examined by the clients.
   endGame();
   $Game::Schedule = schedule($Game::EndGamePause * 1000, 0, "onCyclePauseEnd");
}

function onCyclePauseEnd()
{
   $Game::Cycling = false;

   // Just cycle through the missions for now.
   %search = $Server::MissionFileSpec;
   for (%file = findFirstFile(%search); %file !$= ""; %file = findNextFile(%search)) {
      if (%file $= $Server::MissionFile) {
         // Get the next one, back to the first if there
         // is no next.
         %file = findNextFile(%search);
         if (%file $= "")
           %file = findFirstFile(%search);
         break;
      }
   }
   loadMission(%file);

}


//-----------------------------------------------------------------------------
// GameConnection Methods
// These methods are extensions to the GameConnection class. Extending
// GameConnection make is easier to deal with some of this functionality,
// but these could also be implemented as stand-alone functions.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function GameConnection::onClientEnterGame(%this)
{
   //Check to see if more than two players are connected already...
   echo("Player: " @ ClientGroup.getCount() @ "entering game...");
//   if ( ClientGroup.getCount() > 2 )
//   {
//      echo("Maximum # of clients already connected...");
//      %cl = ClientGroup.getObject( %this );
//      commandToClient(%cl, 'GameEnd');
//	  //This will make the client hang.  Need to improve it slightly...
//	  return;
//   }

   commandToClient(%this, 'SyncClock', $Sim::Time - $Game::StartTime);

   // Create a new camera object.
   %this.camera = new Camera() {
      dataBlock = Observer;
   };
   MissionCleanup.add( %this.camera );
   %this.camera.scopeToClient(%this);

   // Setup game parameters, the onConnect method currently starts
   // everyone with a 0 score.
   %this.score = 0;

   // Create a player object.
   %this.spawnPlayer();
}

function GameConnection::onClientLeaveGame(%this)
{
   if (isObject(%this.camera))
      %this.camera.delete();
   if (isObject(%this.player))
      %this.player.delete();
}


//-----------------------------------------------------------------------------

function GameConnection::onLeaveMissionArea(%this)
{
   // The control objects invoked this method when they
   // move out of the mission area.
}

function GameConnection::onEnterMissionArea(%this)
{
   // The control objects invoked this method when they
   // move back into the mission area.
}


//-----------------------------------------------------------------------------

function GameConnection::onDeath(%this, %sourceObject, %sourceClient, %damageType, %damLoc)
{
   // Clear out the name on the corpse
   %this.player.setShapeName("");
   
	
   
   //Clear out this objects score.
   //This is done by awarding them the inverse of their current score.
   //Temporary hack to allow the GUI update to occur without an additional
   //method. Will need to change with scoreboard additions.
   %this.player.awardPoints(-%this.player.getCurrentScore());

   // Switch the client over to the death cam and unhook the player object.
   if (isObject(%this.camera) && isObject(%this.player)) {
      %this.camera.setMode("Corpse",%this.player);
      %this.setControlObject(%this.camera);
   }
   %this.player = 0;

   //DSD change:
   //dumping the built in scoring system, totally useless (and poorly documented)
   //for what we need. Will still display a 'you're an idiot' message on suicide.
   // Doll out points and display an appropriate message
   if (%damageType $= "Suicide" || %sourceClient == %this) {
      %this.incScore(-1);
      messageAll('MsgClientKilled','%1 takes his own life!',%this.name);
   }
   
//   else {
//      %sourceClient.incScore(1);
//      messageAll('MsgClientKilled','%1 gets nailed by %2!',%this.name,%sourceClient.name);
//      if (%sourceClient.score >= $Game::EndGameScore)
//         cycleGame();
//   }
}


//-----------------------------------------------------------------------------

function GameConnection::spawnPlayer(%this)
{
   // Combination create player and drop him somewhere
   %spawnPoint = pickSpawnPoint();
   %this.createPlayer(%spawnPoint);
}   


//-----------------------------------------------------------------------------

function GameConnection::createPlayer(%this, %spawnPoint)
{
   if (%this.player > 0)  {
      // The client should not have a player currently
      // assigned.  Assigning a new one could result in 
      // a player ghost.
      error( "Attempting to create an angus ghost!" );
   }

   // Create the player object
   %player = new Player() {
      dataBlock = PlayerBody;
      client = %this;
   };
   MissionCleanup.add(%player);

   // Player setup...
   %player.setTransform(%spawnPoint);
   %player.setShapeName(%this.name);
   
   //Register Player in EnemyContainer...
   %player.registerPlayerWithEnemyContainer();

   // Starting equipment
   %player.setInventory(Machinegun,1);
   %player.setInventory(Shotgun,1);
   %player.setInventory(Rocketlauncher, 1);
   %player.setInventory(PulseLaser, 1);
   %player.setInventory(LaserBeam, 1);
   //%player.setInventory(AntWeapon, 1);
   //18 Oct 2005	RCBJ
   //Lots of ammo for testing purposes...
   //22 Oct 2005	TL
   //will need to update player.cs if you want ammo inventory greater than 500
   //%player.setInventory(BallisticAmmo,100000);
   
   %player.setInventory(BallisticAmmo,50000);
   %player.setInventory(EnergyAmmo,200);
   //%player.mountImage(ShotgunImage,0);
   %player.mountImage(LaserBeamImage,0);
   //%player.mountImage(MachinegunImage,0);
   //%player.mountImage(PulseLaserImage,0);
   //%player.mountImage(AntWeaponBallisticImage, 0);
   //%player.mountImage(RocketLauncherImage,0);

   // Update the camera to start with the player
   %this.camera.setTransform(%player.getEyeTransform());

   // Give the client control of the player
   %this.player = %player;
   %this.setControlObject(%player);

   //Update the health display.  
   %this.setHealthAmountHud(100);

   //Update the ammo counter.
   %this.setAmmoAmountHud(%player.getInventory(BallisticAmmo));
   
}

function pickSpawnPoint() 
{
   %groupName = "MissionGroup/PlayerDropPoints";
   %group = nameToID(%groupName);

   if (%group != -1) {
      %count = %group.getCount();
      if (%count != 0) {
         %index = getRandom(%count-1);
         %spawn = %group.getObject(%index);
         return %spawn.getTransform();
      }
      else
         error("No spawn points found in " @ %groupName);
   }
   else
      error("Missing spawn points group " @ %groupName);

   // Could be no spawn points, in which case we'll stick the
   // player at the center of the world.
   return "0 0 300 1 0 0 0";
}
