//File: 	health.cs
//Author:	Base FPS example file.
//Date:		19 Oct 2005
//Notes:
//
//	19 Oct 2005	MP
//	- Modified to support text based player health and ammo.
//
//  11 Nov 2005 TL
//	- Modified Health Patches so they repair based on percentage of Player's max HP as stated in GDD
//  - added the 3 different levels of health patches
// Inventory items.  These objects rely on the item & inventory support
// system defined in item.cs and inventory.cs

//-----------------------------------------------------------------------------
// Health kits can be added to your inventory and used to heal up.
//-----------------------------------------------------------------------------

datablock ItemData(HealthKit)
{
   // Mission editor category, this datablock will show up in the
   // specified category under the "shapes" root category.
   category = "Health";

   // Basic Item properties
   shapeFile = "~/data/shapes/items/healthKit.dts";
   mass = 1;
   friction = 1;
   elasticity = 0.3;

   // Dynamic properties defined by the scripts
   pickupName = "a health kit";
   repairAmount = 50;
};

function HealthKit::onUse(%this,%user)
{
   // Apply some health to whoever uses it, the health kit is only
   // used if the user is currently damaged.
   if (%user.getDamageLevel() != 0) {
      %user.decInventory(%this,1);

   //Update health display.
   if (( (100 - %user.getDamageLevel() ) + %this.repairAmount) > 100)
		%user.client.setHealthAmountHud(100);
   else
		%user.client.setHealthAmountHud( (100 - %user.getDamageLevel() ) + %this.repairAmount);

      %user.applyRepair(%this.repairAmount);
      if (%user.client)
         messageClient(%user.client, 'MsgHealthKitUsed', '\c2Health Kit Applied');
   }
}


//-----------------------------------------------------------------------------
// Health Patchs cannot be picked up and are not meant to be added to 
// inventory.  Health is applied automatically when an objects collides
// with a patch.
//-----------------------------------------------------------------------------

datablock ItemData(HealthPatch)
{
   // Mission editor category, this datablock will show up in the
   // specified category under the "shapes" root category.
   category = "Health";

   // Basic Item properties
   shapeFile = "~/data/shapes/items/healthPatch.dts";
   mass = 1;
   friction = 1;
   elasticity = 0.3;

   // Dynamic properties defined by the scripts
   
   //11 Nov 2005 --TL
   //changed to percentage of maxDamage (HP)
   
   repairPercentage = 0.1;
   maxInventory = 0; // No pickup or throw
};

function HealthPatch::onCollision(%this,%obj,%col)
{
   // Apply health to colliding object if it needs it.
   // Works for all shapebase objects.
    echo("col: " @ %col.getMaxDamage());
   %repair = %col.getMaxDamage() * %this.repairPercentage;
   echo("Repairing: " @ %col.getDamageLevel());
   
   if (%col.getDamageLevel() != 0 && %col.getState() !$= "Dead" ) {
 
   //Update the health display.
   if (( (100 - %col.getDamageLevel()  + %repair)) > 100)
	  %col.client.setHealthAmountHud(100);
   else
	  %col.client.setHealthAmountHud( (100 - %col.getDamageLevel() + %repair));

      %col.applyRepair(%repair);
      %obj.respawn();
      if (%col.client)
         messageClient(%col.client, 'MsgHealthPatchUsed', '\c2Health Patch Applied');
   }
}


datablock ItemData(MidHealthPatch : HealthPatch)
{
  className = "HealthPatch";
  repairPercentage = 0.5;
};

datablock ItemData(FullHealthPatch : HealthPatch)
{
  className = "HealthPatch";
  repairPercentage = 1.0;
};



