datablock ParticleData(LaserBeamParticle)
{
   textureName          = "~/data/shapes/laserbeam/smokeParticle";

   dragCoeffiecient     = 0.0;
   gravityCoefficient   = 0.0;   // rises slowly
   inheritedVelFactor   = 0.00;

   lifetimeMS           = 250;  // lasts 0.7 second
   lifetimeVarianceMS   = 0;   //

   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "1.00 0.00 0.00 1.0";
   colors[1]     = "1.00 0.00 0.00 1.0";
   colors[2]     = "0 0 0 0";

   sizes[0]      = 0.25;
   sizes[1]      = 0.5;
   sizes[2]      = 1.0;

   times[0]      = 0.0;
   times[1]      = 0.3;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(LaserBeamEmitter)
{
   ejectionPeriodMS = 10;
   periodVarianceMS = 5;

   ejectionVelocity = 0.25;
   velocityVariance = 0.10;

   thetaMin         = 0.0;
   thetaMax         = 90.0;  

   particles = LaserBeamParticle;
};


//-----------------------------------------------------------------------------
// Projectile Explosion

datablock ParticleData(LaserBeamExplosionParticle)
{
   dragCoefficient      = 2;
   gravityCoefficient   = 0.2;
   inheritedVelFactor   = 0.2;
   constantAcceleration = 0.0;
   lifetimeMS           = 750;
   lifetimeVarianceMS   = 150;
   textureName          = "~/data/shapes/laserbeam/smokeParticle";
   colors[0]     = "1.00 0.00 0.00 1.0";
   colors[1]     = "0.26 0.00 0.00 0.0";
   sizes[0]      = 0.5;
   sizes[1]      = 1.0;
};

datablock ParticleEmitterData(LaserBeamExplosionEmitter)
{
   ejectionPeriodMS = 7;
   periodVarianceMS = 0;
   ejectionVelocity = 2;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 60;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "LaserBeamExplosionParticle";
};

datablock ExplosionData(LaserBeamExplosion)
{
//   particleEmitter = laserBeamExplosionEmitter; //just glow, no cloud
//   emitter = laserBeamExplosionEmitter;
   particleDensity = 40;
   particleRadius = 1;

   faceViewer     = true;
//   explosionShape = "~/data/shapes/explosions/red_yellow/biggest_explosion.dts";
   explosionScale = "1 1 1";

   shakeCamera = false;
   camShakeFreq = "10.0 11.0 10.0";
   camShakeAmp = "5.0 5.0 5.0";
   camShakeDuration = 0.5;
   camShakeRadius = 100.0;

   // Dynamic light
   lightStartRadius = 5.0;
   lightEndRadius = 2.0;
   lightStartColor = "0.4 0.4 0.4";
   lightEndColor = "0.1 0.1 0.1";
};


//-----------------------------------------------------------------------------
// Projectile Object

datablock BeamData(LaserBeamProjectile)
{
   //projectileShapeName = "~/data/shapes/crossbow/projectile.dts";
   directDamage        = 50;
   radiusDamage        = 10;
   damageRadius        = 0.5;
   explosion           = LaserBeamExplosion;
//   particleEmitter     = LaserBeamEmitter;

   muzzleVelocity      = 100;
   velInheritFactor    = 0.3; //0 gives better laserfire, but wonky effects when jumping 

   armingDelay         = 0;
   lifetime            = 500;
   fadeDelay           = 500;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = false;
   gravityMod = 0.0;

   hasLight    = false;
   lightRadius = 10.0;
   lightColor  = "1.0 0.0 0.0";

   ammodeduct = 1;
   beamStyle = 1;
   beamStartRadius = 0.1;
   beamEndRadius = 0.0;
   materialList = "~/data/shapes/laserbeam/red.dml";

};
 

function LaserBeamProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal)
{
   if (%obj.sourceObject == %col)
      return;
	%temp = %this.directDamage * %obj.getDamageModifier();
	echo("LaserBeam on Collision" @ %temp);
      if (%col.getType() & $TypeMasks::ShapeBaseObjectType)
      {
         %col.damage(%obj,%pos,%this.directDamage* %obj.getDamageModifier(),"LaserBeam");
      }


   // Radius damage is a support scripts defined in radiusDamage.cs
   radiusDamage(%obj,%pos,%this.damageRadius,%this.radiusDamage,"LaserBeam",0);
}

function LaserBeamProjectile::onExplode()
{
}

function LaserBeamProjectile::damage(%this, %obj, %sourceObject, %position, %damage, %damageType)
{
   %obj.applyDamage(%damage);
}


datablock BeamData(LaserBeamProjectileLev2 : LaserBeamProjectile)
{
   className = "LaserBeamProjectile";
   directDamage        = 100;
   radiusDamage        = 20;
   damageRadius        = 1.0;
   
   ammodeduct = 2;
   beamStyle = 1;
   beamStartRadius = 0.3;
   beamEndRadius = 0.0;
   lightColor  = "0.0 1.0 0.0";
};

datablock BeamData(LaserBeamProjectileLev3 : LaserBeamProjectile)
{
   className = "LaserBeamProjectile";
   directDamage        = 150;
   radiusDamage        = 30;
   damageRadius        = 1.5;
   
   ammodeduct = 3;
   beamStyle = 1;
   beamStartRadius = 0.5;
   beamEndRadius = 0.0;
   lightColor  = "0.0 0.0 1.0";
};

//-----------------------------------------------------------------------------
//LaserBeam Weapon

datablock ItemData(LaserBeam)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/laserbeam/laser.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;

	// Dynamic properties defined by the scripts
	pickUpName = "a LaserBeam";
	image = LaserBeamImage;
    //directUse = 1;
};

function LaserBeam::onUse(%data,%obj)
{
//Override function in weapon to take in to account levels
	//%data = LaserBeam
	//%obj = Player
	
	%level = %obj.getCurrentLaserBeamLevel();
	echo("LaserBeam Level" @ %level);
	//echo(%obj.getClassName());
	//echo("Machinegun switching");
	if (%obj.getMountedImage($WeaponSlot) != %data.image.getId()) 
	{
	  serverPlay3D(WeaponUseSound,%obj.getTransform());
	  if (%level == 1)
	    %obj.mountImage(%data.image, $WeaponSlot);
	  else if(%level == 2)
	    %obj.mountImage(LaserBeamImageLev2, $WeaponSlot);
	  else //(%level == 3)
	    %obj.mountImage(LaserBeamImageLev3, $WeaponSlot);
	  if (%obj.client)
	    messageClient(%obj.client, 'MsgWeaponUsed', '\c0Weapon selected');
	}
}


function fireNextGun(%obj)
{
   if (!isObject(%obj))
      return;

   if(%obj.fireWeapon)
   {
      if(%obj.nextWeaponFire == 0)
      {
         %obj.setImageTrigger(0, true);
         %obj.setImageTrigger(1, false);
      }
      else
      {
         %obj.setImageTrigger(0, false);
         %obj.setImageTrigger(1, true);
      }
   }
   else
   {
      %obj.setImageTrigger(0, false);
      %obj.setImageTrigger(1, false);
   }
}

function ShapeBaseImageData::onFire(%data, %obj, %slot)
{
	%projectile = %data.projectile;
	echo("damage " @%projectile.directDamage);
   
	%amount = %projectile.ammodeduct;
	echo("ammo used " @ %amount);
    %shellcount = %obj.decInventory(EnergyAmmo,%amount);
    
    //Update Energy ammo counter. -MP
   %currentAmmo = %obj.getInventory(EnergyAmmo);
   %obj.client.setAmmoAmountHud(%currentAmmo);

	if(%shellcount == %amount)
	{
   
		%muzzleVector = %obj.getMuzzleVector(%slot);
		%objectVelocity = %obj.getVelocity();
		%muzzleVelocity = VectorAdd(
		    VectorScale(%muzzleVector, %projectile.muzzleVelocity),
			  VectorScale(%objectVelocity, %projectile.velInheritFactor));

   
       %p = new (%data.projectileType)() 
		{
		 //accuFire=true;
         dataBlock        = %data.projectile;
         initialVelocity  = %muzzleVelocity;
         initialDirecti
          = %obj.getMuzzleVector(%slot);
         initialPosition  = %obj.getMuzzlePoint(%slot);
         sourceObject     = %obj;
         sourceSlot       = %slot;
		client          = %obj.client;
		//laserModifier  = %modifier; //damage modifier 
     };
 
   if (isObject(%obj.lastProjectile) && %obj.deleteLastProjectile)
      %obj.lastProjectile.delete();

   	if(%obj.getIsDamageMultiplier() == 1)
	  %p.setDamageModifier(2.0);
	  
   %obj.lastProjectile = %p;
   %obj.deleteLastProjectile = %data.deleteLastProjectile;
   
   MissionCleanup.add(%p);
   }
   return %p;
}

//-----------------------------------------------------------------------------
//LaserBeam Image

datablock ShapeBaseImageData(LaserBeamImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/laserbeam/laser.dts";
   emap = true;
   
   ammo = EnergyAmmo;

   mountPoint = 0;
   eyeOffset = "0.30 0.40 -0.45";
   //offset = "0 0 0";
   correctMuzzleVector = true;
   className = "WeaponImage";

   projectile = LaserBeamProjectile;
   projectileType = Beam;

   usesEnergy = true; 
   useMountEnergy = false;
   minEnergy = 0;//5;
   fireEnergy = 0;//5;
   fireTimeout = 300;

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "Activate";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.5;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the weapon. Calls the fire script which does 
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Fire";
   stateTransitionOnTriggerUp[3]    = "Ready";
   //stateTransitionOnTriggerDown[3]  = "Ready";
   stateTimeoutValue[3]             = 0.0;
   stateFire[3]                     = true;
   stateRecoil[3]                   = NoRecoil;
   stateAllowImageChange[3]         = false;
   stateScript[3]                   = "onFire";
   //stateEnergyDrain[3]              = 50;
   stateSound[3] = "LaserBeamFireSound"; // **** Added in sound tutorial

   // Play the relead animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.01;
   stateAllowImageChange[4]         = false;
//   stateSequence[6]                 = "Reload";
   //playSpeed[6]                     = 0.1;
   stateEjectShell[6]               = true;

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "NoAmmo";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "NoAmmo";
   stateTransitionOnTriggerDown[5]  = "DryFire";

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 0.2;
   stateTransitionOnTimeout[6]      = "Ready";
   stateSequence[6]                 = "Fire";
};

function LaserBeamImage::onFire(%data,%obj,%slot)
{
   Parent::onFire(%data,%obj,%slot);
   %obj.nextWeaponFire = 1;
   schedule(%data.fireTimeout, 0, "fireNextGun", %obj);   
}


function LaserBeamImageLev2::onFire(%data,%obj,%slot)
{
   echo("Firing lev2");
   Parent::onFire(%data,%obj,%slot);
   %obj.nextWeaponFire = 1;
   schedule(%data.fireTimeout, 0, "fireNextGun", %obj);   
}

function LaserBeam2Image::onFire(%data,%obj,%slot)
{
   Parent::onFire(%data,%obj,%slot);
   %obj.nextWeaponFire = 0;
   schedule(%data.fireTimeout, 0, "fireNextGun", %obj);
}

function LaserBeamImage::onTriggerDown(%this, %obj, %slot)
{
}

function LaserBeamImage::onTriggerUp(%this, %obj, %slot)
{
}

datablock ShapeBaseImageData(LaserBeam2Image : LaserBeamImage)
{
   mountPoint = 1;
};

datablock ShapeBaseImageData(LaserBeamImageLev2 : LaserBeamImage)
{
    projectile = LaserBeamProjectileLev2;
    projectileType = Beam;
    className = "LaserBeamImage";  
};

datablock ShapeBaseImageData(LaserBeamImageLev3 : LaserBeamImage)
{
    projectile = LaserBeamProjectileLev3;
    projectileType = Beam;
    className = "LaserBeamImage";
};
