//File: 	machinegun.cs
//Author:	Base FPS example file.
//Date:		19 Oct 2005
//Notes:
//
//	19 Oct 2005	MP
//	- Modified to support text based player health and ammo.
//  
//  23 Oct 2005 TL
//  - Going back to make things conform with Coding Standard
//	- Changed weapon so that it shoots only one bullet
//	- Made reload time slower
//	- No radial damage
//
//	24 Oct 2005 JS
//	- Removed all references to Crossbow datablocks (such as CrossbowReloadSound)
//	- Switched those references to Machinegun
//
//	24 Oct 2005 JS
//	- Sped the fire rate back up
//		(I couldn't tell if I had the crossbow or the machinegun)
//	- Changed .dts file to actual machinegun model
//
//	25 Oct 2005 Js
//	- Took out explosions
//
//  30 Oct 2005 TL
//	- Starting on Lev2
//  - Modified fire rate to give allowance for higher levels
//
//  02 Nov 2005 TL
//	- Overriding Weapon::OnUse to take in to account weapon level
//
//	15 Nov 2005 TL
//	- adding DamageMultiplier in the onFire method which will check if the player has picked up
//    DamageMultiplier, if so, it will set the projectile damage to double
//	- Modified onCollision to modify damage by retrieving the damageModifier which is 1
//	  if the player didn't pick up the upgrade

//-----------------------------------------------------------------------------
// Machinegun weapon. This file contains all the items related to this weapon
// including explosions, ammo, the item and the weapon item image.
// These objects rely on the item & inventory support system defined
// in item.cs and inventory.cs
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Sounds profiles

datablock AudioProfile(MachinegunReloadSound)
{
filename = "~/data/sound/crossbow_reload.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(MachinegunFireSound)
{
filename = "~/data/sound/crossbow_firing.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(MachinegunFireEmptySound)
{
filename = "~/data/sound/crossbow_firing_empty.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(MachinegunExplosionSound)
{
filename = "~/data/sound/crossbow_explosion.ogg";
description = AudioDefault3d;
preload = true;
};

//-----------------------------------------------------------------------------
// Machinegun bolt projectile splash

datablock ParticleData(MachinegunSplashMist)
{
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.05;
   inheritedVelFactor   = 0.0;
   constantAcceleration = 0.0;
   lifetimeMS           = 400;
   lifetimeVarianceMS   = 100;
   useInvAlpha          = false;
   spinRandomMin        = -90.0;
   spinRandomMax        = 500.0;
   textureName          = "~/data/shapes/crossbow/splash";
   
   colors[0]     = "0.7 0.8 1.0 1.0";
   colors[1]     = "0.7 0.8 1.0 0.5";
   colors[2]     = "0.7 0.8 1.0 0.0";
   
   sizes[0]      = 0.5;
   sizes[1]      = 0.5;
   sizes[2]      = 0.8;
   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(MachinegunSplashMistEmitter)
{
   ejectionPeriodMS = 5;
   periodVarianceMS = 0;
   ejectionVelocity = 3.0;
   velocityVariance = 2.0;
   ejectionOffset   = 0.0;
   thetaMin         = 85;
   thetaMax         = 85;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   lifetimeMS       = 250;
   particles = "MachinegunSplashMist";
};

datablock ParticleData( MachinegunSplashParticle )
{
   dragCoefficient      = 1;
   gravityCoefficient   = 0.2;
   inheritedVelFactor   = 0.2;
   constantAcceleration = -0.0;
   lifetimeMS           = 600;
   lifetimeVarianceMS   = 0;
   colors[0]     = "0.7 0.8 1.0 1.0";
   colors[1]     = "0.7 0.8 1.0 0.5";
   colors[2]     = "0.7 0.8 1.0 0.0";
   sizes[0]      = 0.5;
   sizes[1]      = 0.5;
   sizes[2]      = 0.5;
   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData( MachinegunSplashEmitter )
{
   ejectionPeriodMS = 1;
   periodVarianceMS = 0;
   ejectionVelocity = 3;
   velocityVariance = 1.0;
   ejectionOffset   = 0.0;
   thetaMin         = 60;
   thetaMax         = 80;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   orientParticles  = true;
   lifetimeMS       = 100;
   particles = "MachinegunSplashParticle";
};

datablock SplashData(MachinegunSplash)
{
   numSegments = 15;
   ejectionFreq = 15;
   ejectionAngle = 40;
   ringLifetime = 0.5;
   lifetimeMS = 300;
   velocity = 4.0;
   startRadius = 0.0;
   acceleration = -3.0;
   texWrap = 5.0;

   texture = "~/data/shapes/crossbow/splash";

   emitter[0] = MachinegunSplashEmitter;
   emitter[1] = MachinegunSplashMistEmitter;

   colors[0] = "0.7 0.8 1.0 0.0";
   colors[1] = "0.7 0.8 1.0 0.3";
   colors[2] = "0.7 0.8 1.0 0.7";
   colors[3] = "0.7 0.8 1.0 0.0";
   times[0] = 0.0;
   times[1] = 0.4;
   times[2] = 0.8;
   times[3] = 1.0;
};

//-----------------------------------------------------------------------------
// Machinegun bolt projectile particles

datablock ParticleData(MachinegunBoltParticle)
{
   textureName          = "~/data/shapes/particles/smoke";
   dragCoefficient     = 0.0;
   gravityCoefficient   = -0.1;   // rises slowly
   inheritedVelFactor   = 0.0;
   lifetimeMS           = 50;
   lifetimeVarianceMS   = 10;   // ...more or less
   useInvAlpha = false;
   spinRandomMin = -30.0;
   spinRandomMax = 30.0;

   colors[0]     = "0.1 0.1 0.1 1.0";
   colors[1]     = "0.1 0.1 0.1 1.0";
   colors[2]     = "0.1 0.1 0.1 0";

   sizes[0]      = 0.15;
   sizes[1]      = 0.20;
   sizes[2]      = 0.25;

   times[0]      = 0.0;
   times[1]      = 0.3;
   times[2]      = 1.0;
};

datablock ParticleData(MachinegunBubbleParticle)
{
   textureName          = "~/data/shapes/particles/bubble";
   dragCoefficient      = 0.0;
   gravityCoefficient   = -0.25;   // rises slowly
   inheritedVelFactor   = 0.0;
   constantAcceleration = 0.0;
   lifetimeMS           = 1500;
   lifetimeVarianceMS   = 600;    // ...more or less
   useInvAlpha          = false;
   spinRandomMin        = -100.0;
   spinRandomMax        = 100.0;

   colors[0]     = "0.7 0.8 1.0 0.4";
   colors[1]     = "0.7 0.8 1.0 1.0";
   colors[2]     = "0.7 0.8 1.0 0.0";

   sizes[0]      = 0.2;
   sizes[1]      = 0.2;
   sizes[2]      = 0.2;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(MachinegunBoltEmitter)
{
   ejectionPeriodMS = 2;
   periodVarianceMS = 0;

   ejectionVelocity = 0.0;
   velocityVariance = 0.10;

   thetaMin         = 0.0;
   thetaMax         = 90.0;  

   particles = MachinegunBoltParticle;
};

datablock ParticleEmitterData(MachinegunBoltBubbleEmitter)
{
   ejectionPeriodMS = 9;
   periodVarianceMS = 0;

   ejectionVelocity = 1.0;
   ejectionOffset   = 0.1;
   velocityVariance = 0.5;

   thetaMin         = 0.0;
   thetaMax         = 80.0;

   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;  

   particles = MachinegunBubbleParticle;
};


//-----------------------------------------------------------------------------
// Explosion Debris

// Debris "spark" explosion
datablock ParticleData(MachinegunDebrisSpark)
{
   textureName          = "~/data/shapes/particles/fire";
   dragCoefficient      = 0;
   gravityCoefficient   = 0.0;
   windCoefficient      = 0;
   inheritedVelFactor   = 0.5;
   constantAcceleration = 0.0;
   lifetimeMS           = 500;
   lifetimeVarianceMS   = 50;
   spinRandomMin = -90.0;
   spinRandomMax =  90.0;
   useInvAlpha   = false;

   colors[0]     = "0.8 0.2 0 1.0";
   colors[1]     = "0.8 0.2 0 1.0";
   colors[2]     = "0 0 0 0.0";

   sizes[0]      = 0.2;
   sizes[1]      = 0.3;
   sizes[2]      = 0.1;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(MachinegunDebrisSparkEmitter)
{
   ejectionPeriodMS = 20;
   periodVarianceMS = 0;
   ejectionVelocity = 0.5;
   velocityVariance = 0.25;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 90;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   orientParticles  = false;
   lifetimeMS       = 300;
   particles = "MachinegunDebrisSpark";
};

datablock ExplosionData(MachinegunDebrisExplosion)
{
   emitter[0] = MachinegunDebrisSparkEmitter;

   // Turned off..
   shakeCamera = false;
   impulseRadius = 0;
   lightStartRadius = 0;
   lightEndRadius = 0;
};

// Debris smoke trail
datablock ParticleData(MachinegunDebrisTrail)
{
   textureName          = "~/data/shapes/particles/fire";
   dragCoefficient      = 1;
   gravityCoefficient   = 0;
   inheritedVelFactor   = 0;
   windCoefficient      = 0;
   constantAcceleration = 0;
   lifetimeMS           = 800;
   lifetimeVarianceMS   = 100;
   spinSpeed     = 0;
   spinRandomMin = -90.0;
   spinRandomMax =  90.0;
   useInvAlpha   = true;

   colors[0]     = "0.8 0.3 0.0 1.0";
   colors[1]     = "0.1 0.1 0.1 0.7";
   colors[2]     = "0.1 0.1 0.1 0.0";

   sizes[0]      = 0.2;
   sizes[1]      = 0.3;
   sizes[2]      = 0.4;

   times[0]      = 0.1;
   times[1]      = 0.2;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(MachinegunDebrisTrailEmitter)
{
   ejectionPeriodMS = 30;
   periodVarianceMS = 0;
   ejectionVelocity = 0.0;
   velocityVariance = 0.0;
   ejectionOffset   = 0.0;
   thetaMin         = 170;
   thetaMax         = 180;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   //overrideAdvances = false;
   //orientParticles  = true;
   lifetimeMS       = 5000;
   particles = "MachinegunDebrisTrail";
};

// Debris object
datablock DebrisData(MachinegunExplosionDebris)
{
   shapeFile = "~/data/shapes/crossbow/debris.dts";
   emitters = "MachinegunDebrisTrailEmitter";
   explosion = MachinegunDebrisExplosion;
   
   elasticity = 0.6;
   friction = 0.5;
   numBounces = 1;
   bounceVariance = 1;
   explodeOnMaxBounce = true;
   staticOnMaxBounce = false;
   snapOnMaxBounce = false;
   minSpinSpeed = 0;
   maxSpinSpeed = 700;
   render2D = false;
   lifetime = 4;
   lifetimeVariance = 0.4;
   velocity = 5;
   velocityVariance = 0.5;
   fade = false;
   useRadiusMass = true;
   baseRadius = 0.3;
   gravModifier = 0.5;
   terminalVelocity = 6;
   ignoreWater = true;
};


//-----------------------------------------------------------------------------
// Bolt Explosion

datablock ParticleData(MachinegunExplosionSmoke)
{
   textureName          = "~/data/shapes/particles/smoke";
   dragCoeffiecient     = 100.0;
   gravityCoefficient   = 0;
   inheritedVelFactor   = 0.25;
   constantAcceleration = -0.30;
   lifetimeMS           = 1200;
   lifetimeVarianceMS   = 300;
   useInvAlpha =  true;
   spinRandomMin = -80.0;
   spinRandomMax =  80.0;

   colors[0]     = "0.56 0.36 0.26 1.0";
   colors[1]     = "0.2 0.2 0.2 1.0";
   colors[2]     = "0.0 0.0 0.0 0.0";

   sizes[0]      = 4.0;
   sizes[1]      = 2.5;
   sizes[2]      = 1.0;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleData(MachinegunExplosionBubble)
{
   textureName          = "~/data/shapes/particles/bubble";
   dragCoeffiecient     = 0.0;
   gravityCoefficient   = -0.25;
   inheritedVelFactor   = 0.0;
   constantAcceleration = 0.0;
   lifetimeMS           = 1500;
   lifetimeVarianceMS   = 600;
   useInvAlpha          = false;
   spinRandomMin        = -100.0;
   spinRandomMax        =  100.0;

   colors[0]     = "0.7 0.8 1.0 0.4";
   colors[1]     = "0.7 0.8 1.0 0.4";
   colors[2]     = "0.7 0.8 1.0 0.0";

   sizes[0]      = 0.3;
   sizes[1]      = 0.3;
   sizes[2]      = 0.3;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(MachinegunExplosionSmokeEmitter)
{
   ejectionPeriodMS = 10;
   periodVarianceMS = 0;
   ejectionVelocity = 4;
   velocityVariance = 0.5;
   thetaMin         = 0.0;
   thetaMax         = 180.0;
   lifetimeMS       = 250;
   particles = "MachinegunExplosionSmoke";
};

datablock ParticleEmitterData(MachinegunExplosionBubbleEmitter)
{
   ejectionPeriodMS = 9;
   periodVarianceMS = 0;
   ejectionVelocity = 1;
   ejectionOffset   = 0.1;
   velocityVariance = 0.5;
   thetaMin         = 0.0;
   thetaMax         = 80.0;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   particles = "MachinegunExplosionBubble";
};

datablock ParticleData(MachinegunExplosionFire)
{
   textureName          = "~/data/shapes/particles/fire";
   dragCoeffiecient     = 100.0;
   gravityCoefficient   = 0;
   inheritedVelFactor   = 0.25;
   constantAcceleration = 0.1;
   lifetimeMS           = 1200;
   lifetimeVarianceMS   = 300;
   useInvAlpha =  false;
   spinRandomMin = -80.0;
   spinRandomMax =  80.0;

   colors[0]     = "0.8 0.4 0 0.8";
   colors[1]     = "0.2 0.0 0 0.8";
   colors[2]     = "0.0 0.0 0.0 0.0";

   sizes[0]      = 1.5;
   sizes[1]      = 0.9;
   sizes[2]      = 0.5;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(MachinegunExplosionFireEmitter)
{
   ejectionPeriodMS = 10;
   periodVarianceMS = 0;
   ejectionVelocity = 0.8;
   velocityVariance = 0.5;
   thetaMin         = 0.0;
   thetaMax         = 180.0;
   lifetimeMS       = 250;
   particles = "MachinegunExplosionFire";
};

datablock ParticleData(MachinegunExplosionSparks)
{
   textureName          = "~/data/shapes/particles/spark";
   dragCoefficient      = 1;
   gravityCoefficient   = 0.0;
   inheritedVelFactor   = 0.2;
   constantAcceleration = 0.0;
   lifetimeMS           = 500;
   lifetimeVarianceMS   = 350;

   colors[0]     = "0.60 0.40 0.30 1.0";
   colors[1]     = "0.60 0.40 0.30 1.0";
   colors[2]     = "1.0 0.40 0.30 0.0";

   sizes[0]      = 0.25;
   sizes[1]      = 0.15;
   sizes[2]      = 0.15;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleData(MachinegunExplosionWaterSparks)
{
   textureName          = "~/data/shapes/particles/bubble";
   dragCoefficient      = 0;
   gravityCoefficient   = 0.0;
   inheritedVelFactor   = 0.2;
   constantAcceleration = 0.0;
   lifetimeMS           = 500;
   lifetimeVarianceMS   = 350;

   colors[0]     = "0.4 0.4 1.0 1.0";
   colors[1]     = "0.4 0.4 1.0 1.0";
   colors[2]     = "0.4 0.4 1.0 0.0";

   sizes[0]      = 0.5;
   sizes[1]      = 0.5;
   sizes[2]      = 0.5;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

datablock ParticleEmitterData(MachinegunExplosionSparkEmitter)
{
   ejectionPeriodMS = 3;
   periodVarianceMS = 0;
   ejectionVelocity = 5;
   velocityVariance = 1;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 180;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   orientParticles  = true;
   lifetimeMS       = 100;
   particles = "MachinegunExplosionSparks";
};

datablock ParticleEmitterData(MachinegunExplosionWaterSparkEmitter)
{
   ejectionPeriodMS = 3;
   periodVarianceMS = 0;
   ejectionVelocity = 4;
   velocityVariance = 4;
   ejectionOffset   = 0.0;
   thetaMin         = 0;
   thetaMax         = 60;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvances = false;
   orientParticles  = true;
   lifetimeMS       = 200;
   particles = "MachinegunExplosionWaterSparks";
};

datablock ExplosionData(MachinegunSubExplosion1)
{
   offset = 0;
   emitter[0] = MachinegunExplosionSmokeEmitter;
   emitter[1] = MachinegunExplosionSparkEmitter;
};

datablock ExplosionData(MachinegunSubExplosion2)
{
   offset = 1.0;
   emitter[0] = MachinegunExplosionSmokeEmitter;
   emitter[1] = MachinegunExplosionSparkEmitter;
};

datablock ExplosionData(MachinegunSubWaterExplosion1)
{
   delayMS   = 100;
   offset    = 1.2;
   playSpeed = 1.5;

   emitter[0] = MachinegunExplosionBubbleEmitter;
   emitter[1] = MachinegunExplosionWaterSparkEmitter;
   
   sizes[0] = "0.75 0.75 0.75";
   sizes[1] = "1.0 1.0 1.0";
   sizes[2] = "0.5 0.5 0.5";
   times[0] = 0.0;
   times[1] = 0.5;
   times[2] = 1.0;
};

datablock ExplosionData(MachinegunSubWaterExplosion2)
{
   delayMS   = 50;
   offset    = 1.2;
   playSpeed = 0.75;

   emitter[0] = MachinegunExplosionBubbleEmitter;
   emitter[1] = MachinegunExplosionWaterSparkEmitter;

   sizes[0] = "1.5 1.5 1.5";
   sizes[1] = "1.5 1.5 1.5";
   sizes[2] = "1.0 1.0 1.0";
   times[0] = 0.0;
   times[1] = 0.5;
   times[2] = 1.0;
};

datablock ExplosionData(MachinegunExplosion)
{
   soundProfile = MachinegunExplosionSound;
   lifeTimeMS = 1200;

   // Volume particles
   //particleEmitter = MachinegunExplosionFireEmitter;
   //particleDensity = 75;
   //particleRadius = 2;

   // Point emission
   //emitter[0] = MachinegunExplosionSmokeEmitter;
   //emitter[1] = MachinegunExplosionSparkEmitter;

   // Sub explosion objects
   //subExplosion[0] = MachinegunSubExplosion1;
   //subExplosion[1] = MachinegunSubExplosion2;
   
   // Camera Shaking
   //shakeCamera = false;
   //camShakeFreq = "10.0 11.0 10.0";
   //camShakeAmp = "1.0 1.0 1.0";
   //camShakeDuration = 0.0;
   //camShakeRadius = 10.0;

   // Exploding debris
   //debris = MachinegunExplosionDebris;
   //debrisThetaMin = 0;
   //debrisThetaMax = 60;
   //debrisPhiMin = 0;
   //debrisPhiMax = 360;
   //debrisNum = 6;
   //debrisNumVariance = 2;
   //debrisVelocity = 1;
   //debrisVelocityVariance = 0.5;
   
   // Impulse
   //impulseRadius = 10;
   //impulseForce = 15;


   // Dynamic light
   //lightStartRadius = 6;
   //lightEndRadius = 3;
   //lightStartColor = "0.5 0.5 0";
   //lightEndColor = "0 0 0";
};

datablock ExplosionData(MachinegunWaterExplosion)
{
   soundProfile = MachinegunExplosionSound;

   // Volume particles
   particleEmitter = MachinegunExplosionBubbleEmitter;
   particleDensity = 375;
   particleRadius = 2;


   // Point emission
   emitter[0] = MachinegunExplosionBubbleEmitter;
   emitter[1] = MachinegunExplosionWaterSparkEmitter;

   // Sub explosion objects
   subExplosion[0] = MachinegunSubWaterExplosion1;
   subExplosion[1] = MachinegunSubWaterExplosion2;
   
   // Camera Shaking
   shakeCamera = true;
   camShakeFreq = "8.0 9.0 7.0";
   camShakeAmp = "3.0 3.0 3.0";
   camShakeDuration = 1.3;
   camShakeRadius = 20.0;

   // Exploding debris
   debris = MachinegunExplosionDebris;
   debrisThetaMin = 0;
   debrisThetaMax = 60;
   debrisPhiMin = 0;
   debrisPhiMax = 360;
   debrisNum = 6;
   debrisNumVariance = 2;
   debrisVelocity = 0.5;
   debrisVelocityVariance = 0.2;
   
   // Impulse
   impulseRadius = 10;
   impulseForce = 15;

   // Dynamic light
   lightStartRadius = 6;
   lightEndRadius = 3;
   lightStartColor = "0 0.5 0.5";
   lightEndColor = "0 0 0";
};

//-----------------------------------------------------------------------------
// Projectile Object

datablock ProjectileData(MachinegunProjectile)
{
   //projectileShapeName = "~/data/shapes/crossbow/projectile.dts";
   directDamage        = 100;  //changed - upping to 20 (MP)
   radiusDamage        = 0;  //changed
   damageRadius        = 0.0;
   //explosion           = MachinegunExplosion;
   //waterExplosion      = MachinegunWaterExplosion;

   particleEmitter     = MachinegunBoltEmitter;
   //particleWaterEmitter= MachinegunBoltBubbleEmitter;

   //splash              = MachinegunSplash;

   muzzleVelocity      = 100;  //changed
   velInheritFactor    = 0.3;

   armingDelay         = 0;
   lifetime            = 5000;
   fadeDelay           = 5000;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = false;  //changed
   gravityMod = 0.10;  //changed

   hasLight    = true;
   lightRadius = 3.0;
   lightColor  = "1.0 0.0 0.0";   //"0.45 0.72 0.92"; //changed

   hasWaterLight     = false;
   waterLightColor   = "0 0.5 0.5";
};

function MachinegunProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal)
{
   // Apply damage to the object all shape base objects
   // echo("Projectile: " @ %obj.getClassName());
  // echo("Doing Damage: " @ %this.directDamage* %obj.getDamageModifier());
   if (%col.getType() & $TypeMasks::ShapeBaseObjectType)
      %col.damage(%obj,%pos,%this.directDamage * %obj.getDamageModifier(),"MachinegunBolt");

   // Radius damage is a support scripts defined in radiusDamage.cs
   // Push the contact point away from the contact surface slightly
   // along the contact normal to derive the explosion center. -dbs
   radiusDamage
     (%obj, VectorAdd(%pos, VectorScale(%normal, 0.01)),
      %this.damageRadius,%this.radiusDamage,"Radius",40);
}

//02 Nov 2005 TL
//Level 2 Projectile

datablock ProjectileData(MachinegunProjectileLev2 : MachinegunProjectile)
{
	className = "MachinegunProjectile";
	directDamage        = 5;  //changed
   //explosion           = MachinegunExplosion;
   //waterExplosion      = MachinegunWaterExplosion;

muzzleVelocity      = 200;  //changed
 
lightRadius = 10.0;
lightColor  = "0.0 1.0 0.0"; //changed

hasWaterLight     = true;
waterLightColor   = "1.0 0.0 0.0";
};

//02 Nov 2005 TL
//Level 3 Projectile

datablock ProjectileData(MachinegunProjectileLev3 : MachinegunProjectile)
{
classname = "MachinegunProjectile";
directDamage        = 5;  //changed
   //explosion           = MachinegunExplosion;
   //waterExplosion      = MachinegunWaterExplosion;

muzzleVelocity      = 300;  //changed
 
lightRadius = 10.0;
lightColor  = "0.0 0.0 1.0"; //changed

hasWaterLight     = true;
waterLightColor   = "1.0 0.0 0.0";
};
//-----------------------------------------------------------------------------
// Ammo Item

datablock ItemData(BallisticAmmo)
{
   // Mission editor category
   category = "Ammo";

   // Add the Ammo namespace as a parent.  The ammo namespace provides
   // common ammo related functions and hooks into the inventory system.
   className = "Ammo";

   // Basic Item properties
   shapeFile = "~/data/shapes/crossbow/ammo.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;

	// Dynamic properties defined by the scripts
	pickUpName = "ballistic ammo";
    maxInventory = 500;
};


//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the MachinegunImage is used.

datablock ItemData(Machinegun)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/machinegun/machinegun.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;
   

	// Dynamic properties defined by the scripts
	pickUpName = "a machinegun";
	image = MachinegunImage;
};

function Machinegun::onUse(%data,%obj)
{
	//Override function in weapon to take in to account levels
	//%data = Machinegun
	//%obj = Player
	
	%level = %obj.getCurrentMachineGunLevel();
	echo("Machinegun Level" @ %level);
	//echo(%obj.getClassName());
	//echo("Machinegun switching");
	if (%obj.getMountedImage($WeaponSlot) != %data.image.getId()) 
	{
	  serverPlay3D(WeaponUseSound,%obj.getTransform());
	  if (%level == 1)
	    %obj.mountImage(%data.image, $WeaponSlot);
	  else if(%level == 2)
	    %obj.mountImage(MachinegunImageLev2, $WeaponSlot);
	  else //(%level == 3)
	    %obj.mountImage(MachinegunImageLev3, $WeaponSlot);
	  if (%obj.client)
	    messageClient(%obj.client, 'MsgWeaponUsed', '\c0Weapon selected');
	}
}


//--------------------------------------------------------------------------
// Machinegun image which does all the work.  Images do not normally exist in
// the world, they can only be mounted on ShapeBase objects.

datablock ShapeBaseImageData(MachinegunImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/machinegun/machinegun.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   eyeOffset = "0.1 0.4 -0.6";

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.  
   correctMuzzleVector = false;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = Machinegun;
   ammo = BallisticAmmo;
   projectile = MachinegunProjectile;
   projectileType = Projectile;
   
   //Projectile spread: increasing the value will make the radius bigger
   projectileSpread = 5/1000;
   isShotGun = true;
   
   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.6;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the weapon. Calls the fire script which does 
   // the actual work.
   // Note to self: changing this value will affect how long it takes to complete shot
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 0.02;//0.7;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";
   stateSound[3]                    = MachinegunFireSound;

   // Play the relead animation, and transition into
   // Note to self: changing this value will affect how long it takes to do the reload animation
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.1; //0.02 .3
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;
   stateSound[4]                    = MachinegunReloadSound;

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "NoAmmo";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "NoAmmo";
   stateTransitionOnTriggerDown[5]  = "DryFire";

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 1.0;
   stateTransitionOnTimeout[6]      = "NoAmmo";
   stateSound[6]                    = MachinegunFireEmptySound;
};



//-----------------------------------------------------------------------------

function MachinegunImage::onFire(%this, %obj, %slot)
{
	//%obj = player
   %projectile = %this.projectile;
	//echo (%obj.getClassName());
	//echo ("Slot " @ %slot);
	
	//for (%i = 0; %i <8; %i++)
	//{
	//%curWeapon =  %obj.getMountedImage(%i).item.getName();
	//echo ("Weapon " @ %i @ " = " @ %curWeapon); 
	//}
   // Tries to decrement inventory ammo by. The image's ammo state is update
   // automatically by the ammo inventory hooks.
   //if(%this.isShotGun)
   %shellcount = %obj.decInventory(%this.ammo,1);

	//Update ballistic ammo counter. -MP
   %currentAmmo = %obj.getInventory(%this.ammo);
   %obj.client.setAmmoAmountHud(%currentAmmo);
   
   //Spread
   %spread = %this.projectileSpread;
   %spread = %spread $= ""?0.001 : %spread;
   %spread = %spread <= 0 ? 0.001: %spread;
   
   echo("Velocity is " @ %projectile.muzzleVelocity);
   // Determine initial projectile velocity based on the 
   // gun's muzzle point and the object's current velocity
   %muzzleVector = %obj.getMuzzleVector(%slot);
   %objectVelocity = %obj.getVelocity();
   %muzzleVelocity = VectorAdd(
      VectorScale(%muzzleVector, %projectile.muzzleVelocity),
      VectorScale(%objectVelocity, %projectile.velInheritFactor));

   // Determine our random x, y and z points in our spread circle and create 
   // a spread matrix.
	%x = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
	%y = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
	%z = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
	%mat = MatrixCreateFromEuler(%x @ " " @ %y @ " " @ %z);
	
	%muzzleVelocity = MatrixMulVector(%mat, %muzzleVelocity);
   // Create the projectile object
   %p = new (%this.projectileType)() {
      dataBlock        = %projectile;
      initialVelocity  = %muzzleVelocity;
      initialPosition  = %obj.getMuzzlePoint(%slot);
      sourceObject     = %obj;
      sourceSlot       = %slot;
      client           = %obj.client;
   };
  // echo("Modifying: " @%obj.getIsDamageMultiplier());
   if(%obj.getIsDamageMultiplier() == 1)
	%p.setDamageModifier(2.0);
   MissionCleanup.add(%p);
   
   return %p;
}

//02 Nov 2005 TL
//Level 2 Weapon Image
datablock ShapeBaseImageData(MachinegunImageLev2 : MachinegunImage)
{

className = "WeaponImage";

   // Projectile && Ammo.
item = Machinegun;
ammo = BallisticAmmo;
projectile = MachinegunProjectileLev2;
projectileType = Projectile;
   
   //Projectile spread: increasing the value will make the radius bigger
projectileSpread = 10/1000;

   
stateTimeoutValue[3]             = 0.1;//0.7;
   
stateTimeoutValue[4]             = 0.1; //0.02 .3
};

function MachinegunImageLev2::onFire(%this, %obj, %slot)
{
   %projectile = %this.projectile;

   if(%this.isShotGun)
   %shellcount = %obj.decInventory(%this.ammo,1);

	//Update ballistic ammo counter. -MP
   %currentAmmo = %obj.getInventory(%this.ammo);
   %obj.client.setAmmoAmountHud(%currentAmmo);
   
   //Spread
   %spread = %this.projectileSpread;
   %spread = %spread $= ""?0.001 : %spread;
   %spread = %spread <= 0 ? 0.001: %spread;
   
   echo("Velocity is " @ %projectile.muzzleVelocity);
   // Determine initial projectile velocity based on the 
   // gun's muzzle point and the object's current velocity
   %muzzleVector = %obj.getMuzzleVector(%slot);
   %objectVelocity = %obj.getVelocity();
   %muzzleVelocity = VectorAdd(
      VectorScale(%muzzleVector, %projectile.muzzleVelocity),
      VectorScale(%objectVelocity, %projectile.velInheritFactor));

   // Determine our random x, y and z points in our spread circle and create 
   // a spread matrix.
	%x = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
	%y = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
	%z = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
	%mat = MatrixCreateFromEuler(%x @ " " @ %y @ " " @ %z);
	
	%muzzleVelocity = MatrixMulVector(%mat, %muzzleVelocity);
   // Create the projectile object
   %p = new (%this.projectileType)() {
      dataBlock        = %projectile;
      initialVelocity  = %muzzleVelocity;
      initialPosition  = %obj.getMuzzlePoint(%slot);
      sourceObject     = %obj;
      sourceSlot       = %slot;
      client           = %obj.client;
   };
   if(%obj.getIsDamageMultiplier() == 1)
	%p.setDamageModifier(2.0);
   MissionCleanup.add(%p);
   
   return %p;
}

//02 Nov 2005 TL
//Level 2 Weapon Image

datablock ShapeBaseImageData(MachinegunImageLev3: MachinegunImage)
{
className = "WeaponImage";

   // Projectile && Ammo.
item = Machinegun;
ammo = BallisticAmmo;
projectile = MachinegunProjectileLev3;
projectileType = Projectile;
   
   //Projectile spread: increasing the value will make the radius bigger
projectileSpread = 10/1000;

   
stateTimeoutValue[3]             = 0.05;//0.7;
   
stateTimeoutValue[4]             = 0.05; //0.02 .3
};

function MachinegunImageLev3::onFire(%this, %obj, %slot)
{
   %projectile = %this.projectile;

   // Tries to decrement inventory ammo by. The image's ammo state is update
   // automatically by the ammo inventory hooks.
   if(%this.isShotGun)
   %shellcount = %obj.decInventory(%this.ammo,1);

	//Update ballistic ammo counter. -MP
   %currentAmmo = %obj.getInventory(%this.ammo);
   %obj.client.setAmmoAmountHud(%currentAmmo);
   
   //Spread
   %spread = %this.projectileSpread;
   %spread = %spread $= ""?0.001 : %spread;
   %spread = %spread <= 0 ? 0.001: %spread;
   
   echo("Velocity is " @ %projectile.muzzleVelocity);
   // Determine initial projectile velocity based on the 
   // gun's muzzle point and the object's current velocity
   %muzzleVector = %obj.getMuzzleVector(%slot);
   %objectVelocity = %obj.getVelocity();
   %muzzleVelocity = VectorAdd(
      VectorScale(%muzzleVector, %projectile.muzzleVelocity),
      VectorScale(%objectVelocity, %projectile.velInheritFactor));

   // Determine our random x, y and z points in our spread circle and create 
   // a spread matrix.
	%x = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
	%y = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
	%z = (getRandom() - 0.5) * 2 * 3.1415926 * %spread;
	%mat = MatrixCreateFromEuler(%x @ " " @ %y @ " " @ %z);
	
	%muzzleVelocity = MatrixMulVector(%mat, %muzzleVelocity);
   // Create the projectile object
   %p = new (%this.projectileType)() {
      dataBlock        = %projectile;
      initialVelocity  = %muzzleVelocity;
      initialPosition  = %obj.getMuzzlePoint(%slot);
      sourceObject     = %obj;
      sourceSlot       = %slot;
      client           = %obj.client;
   };
      if(%obj.getIsDamageMultiplier() == 1)
		%p.setDamageModifier(2.0);
   MissionCleanup.add(%p);
   
   return %p;
}